package com.blazinggames.urcl.ui;

// Java imports
import java.awt.*;

// URCL imports
import com.blazinggames.urcl.data.ResMan;
import com.blazinggames.urcl.data.Config;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The display class that URCL games revolve around. 
 * Default display is an error screen.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Display
{
// *********************
// ***** CONSTANTS *****
// *********************

    public static final int ALIGN_LEFT = 0;
    public static final int ALIGN_CENTER = 1;
    public static final int ALIGN_RIGHT = 2;

// *********************
// ***** VARIABLES *****
// *********************

    public String message[] =
    {
        "AN ERROR HAS OCCURRED",
        "Invalid configuration"
    };
    public Font font = new Font("Serif", Font.PLAIN, 16);
    public Color background = Color.black;
    public Color foreground = Color.white;

    protected DisplayHandler _displayHandler = null;
    protected ResMan _res = null;
    protected Config _config = null;
    protected boolean _active = false;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

	/**
	 * Default constructor.
	 */
    public Display()
    {
    }


// ***********************************
// ***** METHODS - Basic running *****
// ***********************************
	/**
	 * Called to initialize the display
	 * 
	 * @param dh display handler that is controlling the display.
	 * @return true if initialized successfully.
	 */
    public boolean initialize(DisplayHandler dh)
    {
        _displayHandler = dh;
        _res = dh.getResMan();
        _config = dh.getConfig();

        return true;
    }

	/**
	 * Starts the display
	 */
    public boolean begin()
    {
        _active = true;
        return true;
    }

	/**
	 * stops the display
	 * @return true if successfull
	 */
    public boolean end()
    {
        _active = false;
        return false;
    }


    // ************************************
    // ***** METHODS - Event handling *****
    // ************************************

	/**
	 * called whenever the mouse has moved.
	 * 
	 * @param x x location of mouse
	 * @param y y location of mouse
	 * @param b state of left mouse button
	 */
    public void mouse_move(int x, int y, boolean b)
    {
        // Placeholder for subclassing
    }

	/**
	 * called when the mouse button has been pressed.
	 * 
	 * @param x x location of mouse
	 * @param y y location of mouse
	 */
    public void mouse_down(int x, int y)
    {
        // placeholder for subclassing
    }

	/**
	 * called when the mouse button has been released.
	 * 
	 * @param x x location of mouse
	 * @param y y location of mouse
	 */
    public void mouse_up(int x, int y)
    {
        // placeholder for subclassing
    }

	/**
	 * Called when key has been pressed.
	 * 
	 * @param code code for key pressed
	 * @param ch character for key pressed
	 */
    public void key_down(int code, char ch)
    {
        // placeholder for subclassing
    }

	/**
	 * Called when key has been released.
	 * 
	 * @param code code for key pressed
	 * @param ch character for key pressed
	 */
    public void key_up(int code, char ch)
    {
        // placeholder for subclassing
    }

	/**
	 * called whenever a timing interval has passed
	 * 
	 * @param ms time in milliseconds since last call
	 */
    public void tick(int ms)
    {
        // placeholder for subclassing
    }


// ************************************************
// ***** METHODS - Display (and display aids) *****
// ************************************************
	
	/**
	 * renders (draws) the display.
	 * 
	 * @param g graphics context
	 * @param comp component drawing is happening in
	 */
    public synchronized void render(Graphics g, Component comp)
    {
        Dimension d = comp.getSize();
        g.setColor(background);
        g.fillRect(0,0, d.width, d.height);
        g.setFont(font);
        g.setColor(foreground);
        int height = d.height / message.length;
        Rectangle r = new Rectangle(0,0, d.width, height);
        FontMetrics fm = g.getFontMetrics();
        for (int cntr = 0; cntr < message.length; ++cntr)
        {
            Point p = getTextPosition(fm, r, message[cntr], 1);
            drawOutlineText(g, message[cntr], p.x, p.y, background, foreground);
            r.translate(0, height);
        }
    }


	/**
	 * utility method for calculating where to position text to draw
	 * 
	 * @param fm fontMetrics of the font to use
	 * @param r area text should appear in
	 * @param s text that will be displayed
	 * @param align alignment of text
	 * @return point where text should be drawn
	 */
    public Point getTextPosition(FontMetrics fm, Rectangle r, String s, int align)
    {
        int ascent = fm.getAscent();
        int y = r.y + (r.height / 2) + (ascent / 2);
        int x;
        int w = fm.stringWidth(s);
        if (align == ALIGN_LEFT)
            x = r.x;
        else if (align == ALIGN_RIGHT)
            x = r.x + r.width - w;
        else
            x = r.x + ((r.width - w) / 2);

        return new Point(x, y);
    }

	/**
	 * Draws outline text
	 * 
	 * @param g graphics context
	 * @param s string to draw
	 * @param x x location to draw text
	 * @param y y location to draw text
	 * @param colText color of text
	 * @param colOut color of outline
	 */
    public void drawOutlineText(Graphics g, String s, int x, int y, Color colText, Color colOut)
    {
        g.setColor(colOut);
        for (int cntrX = -1; cntrX < 2; ++cntrX)
        {
            for (int cntrY = -1; cntrY < 2; ++cntrY)
            {
                g.drawString(s, x+cntrX, y+cntrY);
            }
        }
        g.setColor(colText);
        g.drawString(s, x, y);
    }

	/**
	 * draws text with a shadow
	 * @param g graphics context
	 * @param s text to draw
	 * @param x x location of text
	 * @param y y location of text
	 * @param colText color of text
	 * @param colShadow color of text shadow 
	 */
    public void drawShadowText(Graphics g, String s, int x, int y, Color colText, Color colShadow)
    {
        g.setColor(colShadow);
        g.drawString(s, x+1, y+1);
        g.setColor(colText);
        g.drawString(s, x, y);
    }

}
